import 'dart:collection';
import 'dart:math';

extension ListExtension<T> on List<T> {
  T getOrNull(int index) {
    if (index >= 0 && this.length > index && this.length > 0) {
      return this[index];
    } else {
      return null;
    }
  }

  T firstOrNull() {
    return getOrNull(0);
  }


  /// returns a list with two swapped items
  List<T> swap(int i, int j) {
    final list = this.toList();
    final aux = list[i];
    list[i] = list[j];
    list[j] = aux;
    return list;
  }

  T getRandom() {
    Random generator = Random();
    final index = generator.nextInt(this.length);
    return this.toList()[index];
  }

  T firstOrNullWhere(bool predicate(T element)) {
    return firstWhere(predicate, orElse: () => null);
  }

  /// get the last element if the list is not empty or return null
  T get lastOrNull => isNotEmpty ? last : null;

  T lastOrDefault(T defaultValue) => lastOrNull ?? defaultValue;

  /// get the first element or provider default
  T firstOrDefault(T defaultValue) => firstOrNull ?? defaultValue;

  ///
  /// Performs the given action on each element on iterable, providing sequential index with the element.
  /// [item] the element on the current iteration
  /// [index] the index of the current iteration
  ///
  /// example:
  /// ["a","b","c"].forEachIndexed((element, index) {
  ///    print("$element, $index");
  ///  });
  /// result:
  /// a, 0
  /// b, 1
  /// c, 2
  void forEachIndexed(void action(T element, int index)) {
    var index = 0;
    for (var element in this) {
      action(element, index++);
    }
  }

  ///
  /**
      [1,2,3,4,5,6].subtract([4,5,6])
      result:
      1,2,3
   */

  ///
  subtract(Iterable<T> other) {
    final set = toSet();
    set.removeAll(other);
    return set;
  }

  bool get isEmptyOrNull => this == null || isEmpty;

  Map<dynamic, dynamic> associate(key(element), value(element)) =>
      Map.fromIterable(this, key: key, value: value);

  /// Returns the first element matching the given [predicate], or `null`
  /// if element was not found.
  T find(predicate(T selector)) {
    for (final element in this) {
      if (predicate(element)) {
        return element;
      }
    }

    return null;
  }
  
}
