import 'dart:ui';

extension StringExtensions on String {

  /// convert string hex to color
  Color get convertToColor {
    try {
      final buffer = StringBuffer();
      if (this.length == 6 || this.length == 7) buffer.write('ff');
      buffer.write(this.replaceFirst('#', ''));
      return Color(int.parse(buffer.toString(), radix: 16));
    } catch (e) {
      return null;
    }
  }

  bool get isNullOrEmpty {
    return this == null || isEmpty;
  }

  /// Returns `true` if this string is not `null` and not empty.
  bool get isNotNullNorEmpty {
    return this != null && isNotEmpty;
  }

  bool get validateEmail => RegExp(
      r"^[a-zA-Z0-9.a-zA-Z0-9.!#$%&'*+-/=?^_`{|}~]+@[a-zA-Z0-9]+\.[a-zA-Z]+")
      .hasMatch(this);

  bool equalsIgnoreCase(String other) =>
      (this == null && other == null) ||
          (this != null && other != null && toLowerCase() == other.toLowerCase());

  /// Returns the string if it is not `null`, or the empty string otherwise
  String get orEmpty => this ?? "";

  /// Parses the string as an double or returns `null` if it is not a number.
  double toDoubleOrNull() => double.tryParse(this);

  /// Parses the string as an int or returns `null` if it is not a number.
  int toIntOrNull() => int.tryParse(this);

  /// Returns true if s is neither null, empty nor is solely made of whitespace characters.
  bool get isNotBlank => this != null && trim().isNotEmpty;

  /// Returns `true` if this string is any of these values: `"true"`, `"yes"`, `"1"`, or if the string is a number and greater than 0, `false` if less than 1. This is also case insensitive.
  bool get asBool {
    var s = this.trim().toLowerCase();
    num n;
    try {
      n = num.parse(s);
    } catch (e) {
      n = -1;
    }
    return s == 'true' || s == 'yes' || n > 0;
  }

  ///
  /**
      String temp = 'HElLo';
      String capitalize = temp.capitalize();
      print(capitalize); // output: Hello
   */

  ///
  String capitalize() {
    return '${this[0].toUpperCase()}${substring(1).toLowerCase()}';
  }

  ///
  /**
      String temp = 'Hello world !';
      String countWords = temp.countWords();
      print(countWords); // output: 3
   */

  ///
  int countWords() {
    var words = trim().split(RegExp(r'(\s+)'));
    return words.length;
  }

  ///
  /**
      String temp = 'flutter@gmail.com';
      String isMail = temp.isMail();
      print('isMail: isMail'); // output: isMail: true
   */

  ///
  bool isMail() {
    var regex = RegExp(r"(^\w+([-+.']\w+)*@\w+([-.]\w+)*\.\w+([-.]\w+)*$)");
    return regex.hasMatch(this);
  }

  ///
  /**
      String temp = '1h*^ello12';
      String onlyNumbers = temp.onlyNumbers();
      print(onlyNumbers); // output: 112
   */

  ///
  String onlyNumbers() {
    var regex = RegExp(r'([^0-9]+)');
    return replaceAll(regex, '');
  }

  ///
  /**
      String temp = '1hello12';
      String removeLetters = temp.removeLetters();
      print('removeLetters: removeLetters'); // output: removeLetters: 112
   */

  ///
  String removeLetters() {
    var regex = RegExp(r'([a-zA-Z]+)');
    return replaceAll(regex, '');
  }

  ///
  /**
      String temp = 'hello';
      String revered = temp.reverse();
      print('revered: $revered'); // output: revered: olleh
   */

  ///

  String reverse() {
    var letters = split('').toList().reversed;
    return letters.reduce((current, next) => current + next);
  }
}
