import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:flutter_extension/flutter_extension.dart';

void main() {
  group('list', () {
    test('getOrNull', () {
      var arr = List();
      expect(arr.getOrNull(0), null);
      expect(arr.firstOrNull(), null);
      arr.add('1');
      arr.add('2');
      expect(arr.getOrNull(0), '1');
      expect(arr.firstOrNull(), '1');
    });
  });
  group('number', () {
    test('intOrDouble', () {
      expect(20.0.intOrDouble, 20);
      expect(20.1.intOrDouble, 20.1);
    });
  });
  group('strings', () {
    test('convertToColor', () {
      expect('#ffffffff'.convertToColor, Colors.white);
      expect('ffffff'.convertToColor, Colors.white);
    });
    test('orEmpty', () {
      String nullStr;
      expect(nullStr.orEmpty, '');
      expect(''.orEmpty, '');
      expect('test'.orEmpty, 'test');
    });

    test('validateEmail', () {
      expect('name@domain.com'.validateEmail, true);
      expect('name@domain'.validateEmail, false);
    });
    test('equalsIgnoreCase', () {
      expect('text'.equalsIgnoreCase('Text'), true);
      expect('text'.equalsIgnoreCase('another'), false);
    });
    test('isNotNullNorEmpty', () {
      expect(''.isNotNullNorEmpty, false);
      String nullStr;
      expect(nullStr.isNotNullNorEmpty, false);
      expect('not empty'.isNotNullNorEmpty, true);
    });
    test('isNullOrEmpty', () {
      expect(''.isNullOrEmpty, true);
      String nullStr;
      expect(nullStr.isNullOrEmpty, true);
      expect('not empty'.isNullOrEmpty, false);
    });

    test('toDoubleOrNull', () {
      expect('20.0'.toDoubleOrNull(), 20.0);
      expect('a20.0'.toDoubleOrNull(), null);
    });
    test('toIntOrNull', () {
      expect('20'.toIntOrNull(), 20);
      expect('a20'.toIntOrNull(), null);
    });
    test('isNotBlank', () {
      expect('hello world'.isNotBlank, true);
      expect('hello world      '.isNotBlank, true);
      expect(''.isNotBlank, false);
      final String test = null;
      expect(test.isNotBlank, false);
    });

    test('asBool', () {
      // strings, it is case insensitive
      expect('true'.asBool, true);
      expect('True'.asBool, true);
      expect('false'.asBool, false);
      expect('False'.asBool, false);
      expect('yes'.asBool, true);
      expect('YES'.asBool, true);
      expect('no'.asBool, false);
      expect('NO'.asBool, false);

      // string numbers
      expect('232'.asBool, true);
      expect('1'.asBool, true);
      expect('0'.asBool, false);
      expect('-1'.asBool, false);
    });
  });
  group('date time', () {
    test('toMilliseconds', () {
      expect(1.toMilliseconds(), Duration(milliseconds: 1));
    });

    test('toSeconds', () {
      expect(1.toSeconds(), Duration(seconds: 1));
    });

    test('toMinutes', () {
      expect(1.toMinutes(), Duration(minutes: 1));
    });

    test('toHours', () {
      expect(1.toHours(), Duration(hours: 1));
    });

    test('toDays', () {
      expect(1.toDays(), Duration(days: 1));
    });
  });
}
